@@ -25,12 +25,8 @@
 #include "pangocairo-private.h"
 #include "pango-impl-utils.h"
 
-#if defined (HAVE_CAIRO_ATSUI)
-#  if defined (HAVE_CORE_TEXT)
-#    include "pangocairo-coretext.h"
-#  else
-#    include "pangocairo-atsui.h"
-#  endif
+#if defined (HAVE_CORE_TEXT) && defined (HAVE_CAIRO_QUARTZ)
+#  include "pangocairo-coretext.h"
 #endif
 #if defined (HAVE_CAIRO_WIN32)
 #  include "pangocairo-win32.h"
@@ -62,6 +58,13 @@ pango_cairo_font_map_default_init (PangoCairoFontMapIface *iface)
  * You generally should only use the #PangoFontMap and
  * #PangoCairoFontMap interfaces on the returned object.
  *
+ * You can override the type of backend returned by using an
+ * environment variable %PANGOCAIRO_BACKEND.  Supported types,
+ * based on your build, are fc (fontconfig), win32, and coretext.
+ * If requested type is not available, NULL is returned. Ie.
+ * this is only useful for testing, when at least two backends
+ * are compiled in.
+ *
  * Return value: (transfer full): the newly allocated #PangoFontMap,
  *               which should be freed with g_object_unref().
  *
@@ -70,23 +73,41 @@ pango_cairo_font_map_default_init (PangoCairoFontMapIface *iface)
 PangoFontMap *
 pango_cairo_font_map_new (void)
 {
+  const char *backend = getenv ("PANGOCAIRO_BACKEND");
+  if (backend && !*backend)
+    backend = NULL;
+#if !GLIB_CHECK_VERSION (2, 35, 3)
   /* Make sure that the type system is initialized */
   g_type_init ();
-
-#if defined(HAVE_CAIRO_ATSUI)
-#if defined(HAVE_CORE_TEXT)
-  return g_object_new (PANGO_TYPE_CAIRO_CORE_TEXT_FONT_MAP, NULL);
-#else
-  return g_object_new (PANGO_TYPE_CAIRO_ATSUI_FONT_MAP, NULL);
 #endif
-#elif defined(HAVE_CAIRO_WIN32)
-  return g_object_new (PANGO_TYPE_CAIRO_WIN32_FONT_MAP, NULL);
-#elif defined(HAVE_CAIRO_FREETYPE)
-  return g_object_new (PANGO_TYPE_CAIRO_FC_FONT_MAP, NULL);
-#else
-  g_assert_not_reached ();
+#if defined(HAVE_CORE_TEXT) && defined (HAVE_CAIRO_QUARTZ)
+  if (!backend || 0 == strcmp (backend, "coretext"))
+    return g_object_new (PANGO_TYPE_CAIRO_CORE_TEXT_FONT_MAP, NULL);
+#endif
+#if defined(HAVE_CAIRO_WIN32)
+  if (!backend || 0 == strcmp (backend, "win32"))
+    return g_object_new (PANGO_TYPE_CAIRO_WIN32_FONT_MAP, NULL);
+#endif
+#if defined(HAVE_CAIRO_FREETYPE)
+  if (!backend || 0 == strcmp (backend, "fc")
+	       || 0 == strcmp (backend, "fontconfig"))
+    return g_object_new (PANGO_TYPE_CAIRO_FC_FONT_MAP, NULL);
+#endif
+  {
+    const char backends[] = ""
+#if defined(HAVE_CORE_TEXT) && defined (HAVE_CAIRO_QUARTZ)
+      " coretext"
+#endif
+#if defined(HAVE_CAIRO_WIN32)
+      " win32"
+#endif
+#if defined(HAVE_CAIRO_FREETYPE)
+      " fontconfig"
+#endif
+      ;
+    g_error ("Unknown $PANGOCAIRO_BACKEND value.\n  Available backends are:%s", backends);
+  }
   return NULL;
-#endif
 }
 
 /**
@@ -100,28 +121,25 @@ pango_cairo_font_map_new (void)
  * or in fact in most of those cases, just use
  * @pango_cairo_font_map_get_default().
  *
- * Return value: (transfer full) : the newly allocated #PangoFontMap
- *               of suitable type which should be freed with
- *               g_object_unref(), or %NULL if the requested cairo
- *               font backend is not supported / compiled in.
+ * Return value: (transfer full) (nullable): the newly allocated
+ *               #PangoFontMap of suitable type which should be freed
+ *               with g_object_unref(), or %NULL if the requested
+ *               cairo font backend is not supported / compiled in.
  *
  * Since: 1.18
  **/
 PangoFontMap *
 pango_cairo_font_map_new_for_font_type (cairo_font_type_t fonttype)
 {
+#if !GLIB_CHECK_VERSION (2, 35, 3)
   /* Make sure that the type system is initialized */
   g_type_init ();
-
+#endif
   switch ((int) fonttype)
   {
-#if defined(HAVE_CAIRO_ATSUI)
+#if defined(HAVE_CORE_TEXT) && defined (HAVE_CAIRO_QUARTZ)
     case CAIRO_FONT_TYPE_QUARTZ:
-#if defined(HAVE_CORE_TEXT)
       return g_object_new (PANGO_TYPE_CAIRO_CORE_TEXT_FONT_MAP, NULL);
-#else
-      return g_object_new (PANGO_TYPE_CAIRO_ATSUI_FONT_MAP, NULL);
-#endif
 #endif
 #if defined(HAVE_CAIRO_WIN32)
     case CAIRO_FONT_TYPE_WIN32:
@@ -136,7 +154,7 @@ pango_cairo_font_map_new_for_font_type (cairo_font_type_t fonttype)
   }
 }
 
-static PangoFontMap *default_font_map = NULL;
+static GPrivate default_font_map = G_PRIVATE_INIT (g_object_unref); /* MT-safe */
 
 /**
  * pango_cairo_font_map_get_default:
@@ -153,23 +171,32 @@ static PangoFontMap *default_font_map = NULL;
  * change the Cairo font backend that the default fontmap
  * uses for example.
  *
- * Return value: (transfer none): the default Cairo fontmap
- *  for Pango. This object is owned by Pango and must not be freed.
+ * Note that since Pango 1.32.6, the default fontmap is per-thread.
+ * Each thread gets its own default fontmap.  In this way,
+ * PangoCairo can be used safely from multiple threads.
+ *
+ * Return value: (transfer none): the default PangoCairo fontmap
+ *  for the current thread. This object is owned by Pango and must not be freed.
  *
  * Since: 1.10
  **/
 PangoFontMap *
 pango_cairo_font_map_get_default (void)
 {
-  if (G_UNLIKELY (!default_font_map))
-    default_font_map = pango_cairo_font_map_new ();
+  PangoFontMap *fontmap = g_private_get (&default_font_map);
 
-  return default_font_map;
+  if (G_UNLIKELY (!fontmap))
+    {
+      fontmap = pango_cairo_font_map_new ();
+      g_private_replace (&default_font_map, fontmap);
+    }
+
+  return fontmap;
 }
 
 /**
  * pango_cairo_font_map_set_default:
- * @fontmap: The new default font map, or %NULL
+ * @fontmap: (nullable): The new default font map, or %NULL
  *
  * Sets a default #PangoCairoFontMap to use with Cairo.
  *
@@ -177,6 +204,12 @@ pango_cairo_font_map_get_default (void)
  * default fontmap uses for example.  The old default font map
  * is unreffed and the new font map referenced.
  *
+ * Note that since Pango 1.32.6, the default fontmap is per-thread.
+ * This function only changes the default fontmap for
+ * the current thread.   Default fontmaps of exisiting threads
+ * are not changed.  Default fontmaps of any new threads will
+ * still be created using pango_cairo_font_map_new().
+ *
  * A value of %NULL for @fontmap will cause the current default
  * font map to be released and a new default font
  * map to be created on demand, using pango_cairo_font_map_new().
@@ -188,15 +221,10 @@ pango_cairo_font_map_set_default (PangoCairoFontMap *fontmap)
 {
   g_return_if_fail (fontmap == NULL || PANGO_IS_CAIRO_FONT_MAP (fontmap));
 
-  if ((PangoFontMap *) fontmap == default_font_map)
-    return;
-
-  if (default_font_map)
-    g_object_unref (default_font_map);
-
   if (fontmap)
     g_object_ref (fontmap);
-  default_font_map = (PangoFontMap *) fontmap;
+
+  g_private_replace (&default_font_map, fontmap);
 }
 
 /**
