@@ -1831,7 +1831,7 @@ pango_fc_font_map_config_changed (PangoFcFontMap *fcfontmap)
 /**
  * pango_fc_font_map_set_config:
  * @fcfontmap: a #PangoFcFontMap
- * @fcconfig: (nullable) a #FcConfig, or %NULL
+ * @fcconfig: (nullable): a #FcConfig, or %NULL
  *
  * Set the FcConfig for this font map to use.  The default value
  * is %NULL, which causes Fontconfig to use its global "current config".
@@ -2347,7 +2347,7 @@ pango_fc_face_describe (PangoFontFace *face)
   FcConfigSubstitute (NULL, match_pattern, FcMatchPattern);
   FcDefaultSubstitute (match_pattern);
 
-  if (fcface->family && fcface->family->fontmap)
+  if (fcface->family->fontmap)
     config = fcface->family->fontmap->priv->config;
 
   result_pattern = FcFontMatch (config, match_pattern, &res);
